/* global __html, FileManager, bliss, DataProviderFacade, LogController, Application */

// 

__html.Builder = (function () {
    'use strict';

    function Builder(config) {
        this.config = config;
        this.media = {};
    }

    Builder.prototype.buildSite = function (fso, project) {
        var changes = project.getChanges();
        var deleted = project.getDeleted();

        this.buildHtml(
            fso,
            {
                model: project.model,
                collection: changes
            },
            function (pageHtml) {
                return this.processMedia(pageHtml, 'html');
            }.bind(this)
        );

        this.removeHtml(fso, deleted);
    };

    Builder.prototype.buildMedia = function buildMedia(fso, data) {
        if (data.iconSetFiles) {
            Object.keys(data.iconSetFiles).forEach(function (key) {
                fso.write(
                    FileManager.join(this.config.path.assets.fonts, key),
                    data.iconSetFiles[key],
                    'base64'
                );
                this.media[key] = {
                    htmlPath: '@assets.fonts'
                };
            }, this);
        }

        if (data.images) {
            Object.keys(data.images).forEach(function (key) {
                if (data.images[key]) {
                    fso.write(
                        FileManager.join(this.config.path.assets.images, key),
                        data.images[key],
                        data.images[key] === '[DELETED]' ? '' : 'base64'
                    );
                }
                this.media[key] = {
                    htmlPath: '@assets.images',
                    cssPath: pathToRoot(this.config.path.assets.images) + '/' + this.config.path.assets.images
                };
            }, this);
        }

        if (data.thumbnails) {
            data.thumbnails.forEach(function (thumbnail) {
                fso.write(
                    FileManager.join(this.config.path.assets.images, thumbnail.name),
                    thumbnail.data.replace('data:image/png;base64,', ''),
                    'base64'
                );
            }, this);
        }
    };

    Builder.prototype.buildImages = function buildImages(fso, file) {
        fso.move(file, FileManager.join(this.config.path.assets.images, file));
    };

    Builder.prototype.buildCss = function buildCss(fso, file) {
        var content = this.processMedia(fso.read(file), 'css');
        fso.write(FileManager.join(this.config.path.assets.css, file), content);
    };

    Builder.prototype.buildJs = function buildJs(fso, file) {
        fso.move(file, FileManager.join(this.config.path.assets.js, file));
    };

    Builder.prototype.buildProject = function buildEditor(fso, project) {
        fso.write('__editor__/data/project/' + this.config.path.project, JSON.stringify(project.data));
        fso.write('__editor__/data/project/' + this.config.path.cache, JSON.stringify(project.cache));
        fso.write('__editor__/data/project/' + this.config.path.hashes, JSON.stringify(project.hashes));
        fso.write('__editor__/data/project/' + this.config.path.model, JSON.stringify(project.model.data, null, 4));
        fso.write('__editor__/data/project/' + this.config.path.editorConfig, JSON.stringify(project.editor));
    };

    Builder.prototype.buildApp = function buildApp(fso, project) {
        var appPath = FileManager.join('__editor__/app.html');
        var pathToManifest = DataProviderFacade.getCmsInfo().pathToManifest;

        if (fso.exists(pathToManifest)) {
            fso.copy(pathToManifest, FileManager.join('__manifest__', project.editor.version + '.manifest'));
        }

        fso.write(appPath, this.renderApp(project));
    };

    Builder.prototype.buildHtml = function buildHtml(fso, buildData, preprocess) {
        var model = buildData.model,
            collection = buildData.collection;

        var entityKeys = Object.keys(collection),
            pageTree = [],
            postList = [];


        if (entityKeys.length) {
            LogController.time('[CMS] create page tree');
            pageTree = model.allPageData();
            LogController.timeEnd('[CMS] create page tree');

            LogController.time('[CMS] select posts');
            postList = model.allPostData();
            LogController.timeEnd('[CMS] select posts');
        }

        entityKeys.forEach(function (entityKey) {
            var entity = collection[entityKey],
                template = entity.template,
                previewFolder = this.config.path.preview;

            LogController.time('[CMS] Build page - ' + entity.name);

            LogController.time('[CMS] compose page');
            var finalHtml = preprocess(
                (model.getTemplate(template) || ('Empty template: ' + template))
                    .replace(/<export:content>/g, entity.content)
            );
            LogController.timeEnd('[CMS] compose page');

            LogController.time('[CMS] prepare page data');
            var page = this.preparePageData(model, entity);
            LogController.timeEnd('[CMS] prepare page data');

            LogController.time('[CMS] render');
            // base
            this.renderPage({
                root: '',
                template: finalHtml,
                data: { pages: pageTree, posts: postList, page: page }
            }, function (content) {
                fso.write(page.filePath, removeDataId(content));
            });
            LogController.timeEnd('[CMS] render');

            LogController.time('[CMS] render preview');
            // preview
            this.renderPage({
                root: previewFolder,
                template: finalHtml,
                data: { pages: pageTree, posts: postList, page: page }
            }, function (content) {
                fso.write(FileManager.join(previewFolder, page.filePath), content);
            });
            LogController.timeEnd('[CMS] render preview');

            if (page.order === 0) {
                LogController.time('[CMS] create index');
                var indexFile = FileManager.join(page.path, 'index.html');
                // base index
                fso.copy(page.filePath, indexFile);
                // preview index
                fso.copy(
                    FileManager.join(previewFolder, page.filePath),
                    FileManager.join(previewFolder, page.path, 'index.html')
                );
                LogController.timeEnd('[CMS] create index');
            }

            LogController.timeEnd('[CMS] Build page - ' + entity.name);
        }, this);
    };

    Builder.prototype.removeHtml = function removeHtml(fso, collection) {
        Object.keys(collection).forEach(function (filePath) {
            fso.write(filePath, '[DELETED]');
            fso.write(FileManager.join(this.config.path.preview, filePath), '[DELETED]');
        }, this);
    };

    Builder.prototype.preparePageData = function preparePageData(model, entity) {
        var data = model.getEntityData(entity);

        data.parents = model.findParents(entity.key).map(function (i) {
            return model.getEntityData(i);
        });

        if (entity.comments) {
            data.comments = __html.Model.createTree('', entity.comments, function (parentPath, item) {
                var commentData = model.getEntityData(item);
                commentData.name = '';
                return commentData;
            });
        }

        return data;
    };

    Builder.prototype.renderPage = function renderPage(params, callback) {
        LogController.time('[CMS] compute site paths');

        var assetsRoot = pathToRoot(__html.Application.joinPath(params.root, params.data.page.path)),
            pageRoot = pathToRoot(params.data.page.path),
            assets = {};

        Object.keys(this.config.path.assets).forEach(function (k) {
            assets[k] = assetsRoot + '/' + this.config.path.assets[k];
        }, this);

        LogController.timeEnd('[CMS] compute site paths');

        var parse = function parse(content) {
            return renderInternal(argsDef, argsVal, content);
        };

        LogController.time('[CMS] compile');
        var argsDef = '@!(parse, pages, posts, page, assets, pathToRoot)\n',
            argsVal = [
                parse,
                params.data.pages,
                params.data.posts,
                params.data.page,
                assets,
                pageRoot
            ];

        callback(renderInternal(argsDef, argsVal, params.template));
        LogController.timeEnd('[CMS] compile');
    };

    Builder.prototype.renderApp = function renderApp(project) {
        var manifestAttr = project.editor.version ?
                'manifest="../../../manifest/' + project.editor.version + '.manifest"' :
                '',
            def = '@!(manifest)\n',
            val = [manifestAttr];

        return renderInternal(def, val, project.editor.template);
    };

    Builder.prototype.processMedia = function processMedia(content, type) {
        type = type || 'html';
        if (type === 'html') {
            content = content.replace('//fonts.googleapis.com', 'http://fonts.googleapis.com');
        }
        return content.replace(/(url\(['"]?([^\)'"]+)['"]?\))/gi, function (str, match, src) {
            var mediaInfo = this.media[src];
            if (mediaInfo) {
                match = type === 'css' ?
                    'url(' + (mediaInfo.cssPath ? mediaInfo.cssPath  + '/' : '') + src + ')' :
                    (mediaInfo.htmlPath ? mediaInfo.htmlPath + '/' : '') + src;
            } else {
                match = type === 'html' ? src : match;
            }
            return match;
        }.bind(this));
    };

    function renderInternal(def, val, content) {
        return bliss.compile(def + content).apply(null, val).trim();
    }

    function removeDataId(content) {
        return content.replace(/\s*(data-control-id)(=["\'](\d+)["\']|-(\d+))\s*/g, '');
    }

    function pathToRoot(pathFromRoot) {
        if (!pathFromRoot) return '.';
        return pathFromRoot.split('/')
            .filter(function (i) { return i; })
            .map(function () { return '..'; })
            .join('/');
    }

    return Builder;

})();

//